import datetime
import re
from django import forms
from django.core.validators import (
    RegexValidator,
    EmailValidator,
    URLValidator,
    MinLengthValidator,
    MaxLengthValidator
)
from django.core.exceptions import ValidationError


def validate_age(value):
    today = datetime.date.today()
    age = today.year - value.year
    if (today.month, today.day) < (value.month, value.day):
        age -= 1
    if age < 12:
        raise ValidationError("Usia minimal 12 tahun.")


def validate_ipk_range(value):
    if value < 0 or value > 4:
        raise ValidationError("IPK harus dalam rentang 0 hingga 4.")

def validate_blog_url(value):
    validator = URLValidator()
    try:
        validator(value)
    except ValidationError:
        raise ValidationError("Masukkan format URL yang benar.")
    if len(value) < 10:
        raise ValidationError("URL terlalu pendek. Pastikan link lengkap dan valid.")


class UserForm(forms.Form):
    name = forms.CharField(
        label="Nama Pengguna",
        max_length=255,
        required=True,
        validators=[
            RegexValidator(
                r'^[A-Za-z0-9._-]+$',
                message="Nama hanya boleh terdiri dari huruf, angka, titik (.), underscore (_), atau strip (-)."
            )
        ]
    )

    password = forms.CharField(
        label="Password",
        widget=forms.PasswordInput,
        min_length=8,
        required=True,
        help_text="Minimal 8 karakter. Harus mengandung huruf, angka, dan karakter spesial.",
        validators=[
            RegexValidator(
                r'^(?=.*[A-Za-z])(?=.*\d)(?=.*[^\w\s]).{8,}$',
                message="Password harus mengandung minimal 1 huruf, 1 angka, dan 1 karakter spesial."
            )
        ]
    )

    birth_date = forms.DateField(
        label="Tanggal Lahir",
        required=True,
        validators=[validate_age],
        widget=forms.DateInput(attrs={'type': 'date'})
    )

    phone_number = forms.CharField(
        label="Nomor HP",
        required=True,
    )

    email = forms.EmailField(
        label="Email",
        required=True,
        validators=[EmailValidator("Masukkan format email yang benar.")]
    )

    blog_url = forms.URLField(
        label="URL Blog",
        required=True,
        validators=[validate_blog_url],
        error_messages={'invalid': 'Masukkan format URL yang benar.'}
    )

    description = forms.CharField(
        label="Deskripsi Diri",
        required=True,
        widget=forms.Textarea,
        validators=[
            MinLengthValidator(5, message="Minimal 5 karakter."),
            MaxLengthValidator(1000, message="Maksimal 1000 karakter.")
        ]
    )

    npm = forms.CharField(
        label="NPM",
        required=True,
        validators=[
            RegexValidator(
                regex=r'^[0-9]{10}$',
                message="NPM harus 10 digit angka."
            )
        ]
    )

    ipk = forms.DecimalField(
        label="IPK",
        required=True,
        max_digits=4,      
        decimal_places=2,  
        validators=[validate_ipk_range]
    )

    def clean_phone_number(self):
        phone = self.cleaned_data.get("phone_number", "")
        sanitized = re.sub(r'\D', '', phone)
        if not (8 <= len(sanitized) <= 15):
            raise ValidationError("Nomor HP harus terdiri dari 8 hingga 15 digit.")
        return sanitized

    def clean(self):
        cleaned_data = super().clean()
        pwd = cleaned_data.get("password")
        if pwd and "password" in pwd.lower():
            self.add_error("password", "Password tidak boleh mengandung kata 'password'.")
        return cleaned_data
