import datetime
from django.test import TestCase, Client
from django.urls import reverse
from django.core.exceptions import ValidationError
from .forms import UserForm
from .models import CustomUser

class UserFormTests(TestCase):
    def setUp(self):
        self.valid_data = {
            'name': 'TestUser',
            'password': 'StrongPass1!',
            'birth_date': (datetime.date.today() - datetime.timedelta(days=365*15)).isoformat(),  # 15 tahun
            'phone_number': '08123456789',
            'email': 'test@example.com',
            'blog_url': 'https://exampleblog.com',
            'description': 'Deskripsi singkat tentang diri saya.',
            'npm': '1234567890',
            'ipk': '3.50'
        }
    
    def test_valid_form(self):
        form = UserForm(data=self.valid_data)
        self.assertTrue(form.is_valid())

    def test_invalid_age(self):
        data = self.valid_data.copy()
        data['birth_date'] = (datetime.date.today() - datetime.timedelta(days=365*10)).isoformat() 
        form = UserForm(data=data)
        self.assertFalse(form.is_valid())
        self.assertIn("Usia minimal 12 tahun.", form.errors.get("birth_date", []))
    
    def test_password_cannot_contain_word_password(self):
        data = self.valid_data.copy()
        data['password'] = 'mypassword1!'  
        form = UserForm(data=data)
        self.assertFalse(form.is_valid())
        self.assertIn("Password tidak boleh mengandung kata 'password'.", form.errors.get("password", []))
    
    def test_invalid_phone_number_length(self):
        data = self.valid_data.copy()
        data['phone_number'] = '123' 
        form = UserForm(data=data)
        self.assertFalse(form.is_valid())
        self.assertIn("Nomor HP harus terdiri dari 8 hingga 15 digit.", form.errors.get("phone_number", []))
    
    def test_invalid_ipk_range(self):
        data = self.valid_data.copy()
        data['ipk'] = '-1'
        form = UserForm(data=data)
        self.assertFalse(form.is_valid())
        self.assertIn("IPK harus dalam rentang 0 hingga 4.", form.errors.get("ipk", []))
        
        data = self.valid_data.copy()
        data['ipk'] = '4.50'
        form = UserForm(data=data)
        self.assertFalse(form.is_valid())
        self.assertIn("IPK harus dalam rentang 0 hingga 4.", form.errors.get("ipk", []))
    
    def test_invalid_blog_url(self):
        data = self.valid_data.copy()
        data['blog_url'] = 'invalid-url'
        form = UserForm(data=data)
        self.assertFalse(form.is_valid())
        self.assertIn("Masukkan format URL yang benar.", form.errors.get("blog_url", []))
        

        data = self.valid_data.copy()
        data['blog_url'] = 'http://a.com'
        form = UserForm(data=data)
        self.assertTrue(form.is_valid())
    
    def test_invalid_npm(self):
        data = self.valid_data.copy()
        data['npm'] = '12345' 
        form = UserForm(data=data)
        self.assertFalse(form.is_valid())
        self.assertIn("NPM harus 10 digit angka.", form.errors.get("npm", []))
    
    def test_phone_number_sanitization(self):
        data = self.valid_data.copy()
        data['phone_number'] = '+62 812-3456-789'
        form = UserForm(data=data)
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['phone_number'], '628123456789')


class UserViewTests(TestCase):
    def setUp(self):
        self.client = Client()
        self.valid_data = {
            'name': 'TestUser',
            'password': 'StrongPass1!',
            'birth_date': (datetime.date.today() - datetime.timedelta(days=365*15)).isoformat(),
            'phone_number': '08123456789',
            'email': 'testview@example.com',
            'blog_url': 'https://exampleblog.com',
            'description': 'Deskripsi dari test view.',
            'npm': '1234567890',
            'ipk': '3.50'
        }
    
    def test_get_user_form(self):
        response = self.client.get('')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'user_form.html')
    
    def test_post_valid_user_form(self):
        response = self.client.post('', data=self.valid_data)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'user_result.html')
        self.assertTrue(CustomUser.objects.filter(email=self.valid_data['email']).exists())
    
    def test_post_invalid_user_form(self):
        data = self.valid_data.copy()
        data['email'] = 'invalid-email'
        response = self.client.post('', data=data)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'user_form.html')
        self.assertFalse(CustomUser.objects.filter(email=data['email']).exists())


class CustomUserModelTests(TestCase):
    def test_create_custom_user(self):
        user = CustomUser.objects.create_user(
            username='anthony',
            email='modeltest@example.com',
            password='StrongPass1!',
            name='ModelTest',
            birth_date=(datetime.date.today() - datetime.timedelta(days=365*20)),
            phone_number='08123456789',
            blog_url='https://exampleblog.com',
            description='Pengujian model CustomUser.',
            npm='1234567890',
            ipk='3.80'
        )
        self.assertEqual(user.email, 'modeltest@example.com')
        self.assertTrue(user.check_password('StrongPass1!'))

